@@ -1,7 +1,7 @@
 /*
  This file is part of MOST.
 
- Copyright (c) 1991, 1999, 2002 John E. Davis
+ Copyright (c) 1991, 1999, 2002, 2005, 2006, 2007 John E. Davis
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -195,16 +195,18 @@ static void redraw_cmd(void)
    most_redraw_display();
 }
 
-static int read_integer (char *prompt, int *n)
+static int read_integer (char *prompt, MOST_INT *n)
 {
+   int status;
    Most_Mini_Buf[0] = 0;
-
-   if (!most_read_from_minibuffer(prompt, (char *) Most_Mini_Buf))
+   
+   status = most_read_from_minibuffer(prompt, NULL, (char *) Most_Mini_Buf, MOST_MINI_BUF_LEN);
+   if (status < 0)
      return -1;
 
-   if (1 != sscanf((char *) Most_Mini_Buf, "%d", n))
+   if (1 != sscanf((char *) Most_Mini_Buf, MOST_INT_D_FMT, n))
      {
-	most_message ("Expecting integer.", 1);
+	most_message ("Expecting an integer", 1);
 	return -1;
      }
 
@@ -214,7 +216,7 @@ static int read_integer (char *prompt, int *n)
 
 static void goto_line_cmd(void)
 {
-   int n;
+   MOST_INT n;
 
    if (Most_Digit_Arg != (int *) NULL) n = *Most_Digit_Arg;
    else
@@ -222,15 +224,15 @@ static void goto_line_cmd(void)
 	if (-1 == read_integer ("Goto Line: ", &n))
 	  return;
      }
-   most_update_windows (n);
+   most_update_windows ((int)n);
 }
 
 static void goto_percent_cmd(void)
 {
    unsigned char *pos;
-   int n;
+   MOST_INT n;
 
-   if (Most_Digit_Arg != (int *) NULL) n = *Most_Digit_Arg;
+   if (Most_Digit_Arg != NULL) n = *Most_Digit_Arg;
    else
      {
 	if (-1 == read_integer ("Goto Percent: ", &n))
@@ -282,7 +284,7 @@ static void other_window_cmd(void)
 
 static void find_next_cmd(void)
 {
-   int col, line, n = 1;
+   MOST_INT col, line, n = 1;
    unsigned long ofs;
 
    if (Most_Digit_Arg != NULL) n = *Most_Digit_Arg;
@@ -300,31 +302,47 @@ static void find_next_cmd(void)
    Most_Curs_Col = col;
 }
 
-static void search_cmd(void)
+static void find_next_opposite_dir_cmd (void)
 {
-   Most_Search_Dir = 1;
-   if (most_read_from_minibuffer(
-#ifdef SLANG_REGEXP
-				 "Regexp Search: ",
-#else
-				 "Search: ",
-#endif
-				 Most_Search_Str) == -1) return;
+   Most_Search_Dir = -Most_Search_Dir;
+   find_next_cmd ();
+   Most_Search_Dir = -Most_Search_Dir;
+}
+
+static void search_cmd_dir (char *prompt, int dir)
+{
+   char buf[MOST_SEARCH_BUF_LEN];
+
+   if (-1 == most_read_from_minibuffer(prompt, NULL, buf, MOST_SEARCH_BUF_LEN))
+     return;
+
+   Most_Search_Dir = dir;
+   if (*buf)
+     strcpy (Most_Search_Str, buf);    /* no buffer overflow here */
    Most_Curs_Offset = Most_C_Offset;
    find_next_cmd ();
 }
 
+static void search_cmd(void)
+{
+   search_cmd_dir (
+#ifdef SLANG_REGEXP
+		   "Regexp Search: ",
+#else
+		   "Search: ",
+#endif
+		   1);
+}
+
 static void search_back_cmd(void)
 {
-   Most_Search_Dir = -1;
-   if (most_read_from_minibuffer(
+   search_cmd_dir (
 #ifdef SLANG_REGEXP
-				 "Regexp Search Backwards: ",
+		   "Regexp Search Backwards: ",
 #else
-				 "Search Backwards: ",
+		   "Search Backwards: ",
 #endif
-				 Most_Search_Str) == -1) return;
-   find_next_cmd();
+		   -1);
 }
 
 static void help_cmd(void)
@@ -440,6 +458,7 @@ SLKeymap_Function_Type Most_Functions [] =
    A_KEY("exit", most_exit_most),
    A_KEY("find_file", find_file_cmd),
    A_KEY("find_next", find_next_cmd),
+   A_KEY("find_next_other_dir", find_next_opposite_dir_cmd),
    A_KEY("goto_line", goto_line_cmd),
    A_KEY("goto_mark", goto_mark_cmd),
    A_KEY("goto_percent", goto_percent_cmd),
@@ -466,6 +485,13 @@ SLKeymap_Function_Type Most_Functions [] =
 
 SLKeyMap_List_Type *Most_Keymap;
 
+#ifdef IBMPC_SYSTEM
+static void gobble_mouse_cmd(void)
+{
+   SLang_flush_input ();
+}
+#endif
+
 void most_init_keymaps (void)
 {
    char  *err = "Unable to create keymaps!";
@@ -514,7 +540,8 @@ void most_init_keymaps (void)
    SLkm_define_key ("J", (FVOID_STAR) goto_line_cmd, Most_Keymap);
    SLkm_define_key ("L", (FVOID_STAR) most_toggle_lock, Most_Keymap);
    SLkm_define_key ("M", (FVOID_STAR) set_mark_cmd, Most_Keymap);
-   SLkm_define_key ("N", (FVOID_STAR) find_next_cmd, Most_Keymap);
+   SLkm_define_key ("n", (FVOID_STAR) find_next_cmd, Most_Keymap);
+   SLkm_define_key ("N", (FVOID_STAR) find_next_opposite_dir_cmd, Most_Keymap);
    SLkm_define_key ("O", (FVOID_STAR) other_window_cmd, Most_Keymap);
    SLkm_define_key ("Q", (FVOID_STAR) most_exit_most, Most_Keymap);
    SLkm_define_key ("R", (FVOID_STAR) redraw_cmd, Most_Keymap);
@@ -583,6 +610,7 @@ void most_init_keymaps (void)
    SLkm_define_key ("\xE0I", (FVOID_STAR) page_up_cmd, Most_Keymap);
    SLkm_define_key ("\xE0G", (FVOID_STAR) top_of_buffer_cmd, Most_Keymap);
    SLkm_define_key ("\xE0O", (FVOID_STAR) end_of_buffer_cmd, Most_Keymap);
+   SLkm_define_key ("\033[M", (FVOID_STAR) gobble_mouse_cmd, Most_Keymap);
 #endif
    SLkm_define_key ("\t", (FVOID_STAR) page_right_cmd, Most_Keymap);
    SLkm_define_key ("^", (FVOID_STAR) previous_line_cmd, Most_Keymap);
@@ -631,7 +659,7 @@ void most_init_keymaps (void)
    SLkm_define_key ("v", (FVOID_STAR) next_line_cmd, Most_Keymap);
    SLkm_define_key ("w", (FVOID_STAR) toggle_width_cmd, Most_Keymap);
    SLkm_define_key ("x", (FVOID_STAR) most_exit_most, Most_Keymap);
-   if (SLang_Error) most_exit_error (err);
+   if (SLang_get_error()) most_exit_error (err);
 
    (void) most_load_user_keymaps ();
 }
@@ -640,8 +668,9 @@ static void sldo_key (void)
 {
    SLang_Key_Type *key;
 
-   key = SLang_do_key (Most_Keymap, (int (*)(void)) SLang_getkey);
-   SLang_Error = SLKeyBoard_Quit = 0;
+   key = SLang_do_key (Most_Keymap, most_getkey);
+   SLKeyBoard_Quit = 0;
+   SLang_set_error (0);
    Last_Char = SLang_Last_Key_Char;
 
    if ((key == NULL) || (key->f.f == NULL) || (key->type != SLKEY_F_INTRINSIC))
