@@ -1,7 +1,7 @@
 /*
  This file is part of MOST.
 
- Copyright (c) 1991, 1999, 2002, 2005 John E. Davis
+ Copyright (c) 1991, 1999, 2002, 2005, 2006, 2007 John E. Davis
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -17,8 +17,6 @@
  with this program; if not, write to the Free Software Foundation, Inc., 675
  Mass Ave, Cambridge, MA 02139, USA. 
 */
-#define MOST_VERSION "4.10.2"
-
 #include "config.h"
 
 #include <stdio.h>
@@ -40,6 +38,8 @@
 #include <stdarg.h>
 
 #include <slang.h>
+#include "version.h"
+
 #include "jdmacros.h"
 
 #include "most.h"
@@ -49,6 +49,7 @@
 #include "sysdep.h"
 #include "keym.h"
 #include "display.h"
+#include "line.h"
 
 int Most_S_Opt = 0;		       /* squeeze liness */
 int Most_A_Opt = 1;		       /* automatically choose -b if necessary */
@@ -64,12 +65,13 @@ int Most_Captive_Mode;
 #if MOST_HAS_MMAP
 int Most_Disable_MMap	= 0;
 #endif
-int Most_UTF8_Mode = 0;
+
+int Most_UTF8_Mode = -1;	       /* -1:auto, 0:off, 1:on */
 
 static int  Most_Starting_Line;
 char *Most_Program;	/* Program Name (argv[0]) */
 
-char *Most_Version = MOST_VERSION;
+static char *Most_Version = MOST_VERSION_STR;
 
 #ifdef VMS
 # ifndef isalpha
@@ -86,18 +88,20 @@ void most_usage (void)
      fprintf (stderr, " *Note: This executable was compiled against S-Lang %s\n", SLANG_VERSION_STRING);
      
    fprintf (stderr, "Usage:\n");
-   fprintf(stderr, "most [-1Cbckstvw] [+/string] [+line number] [+s] [+d] file...\n");
+   fprintf(stderr, "most [-1Cbcdkstvw] [+/string] [+line number] [+s] [+d] file...\n");
    fputs(" where: -1:  assume VT100 terminal. (VMS only)\n", stderr);
    fputs("        -b:  Startup in binary mode.\n", stderr);
    fputs("        -C:  disable color support\n", stderr);
    fputs("        -c:  Make searches case sensitive.\n", stderr);
-   fputs("        -k:  Kanji mode.\n", stderr);
+   fputs("        -d:  Do not display the \\ wrap marker when wrapping lines.\n", stderr);
+   /* fputs("        -k:  Kanji mode.\n", stderr); */
 #if MOST_HAS_MMAP
    fputs("        -M:  Do not attempt to mmap files.\n", stderr);
 #endif
    fputs("        -s:  Squeeze out excess blank lines.\n", stderr);
    fputs("        -t:  Display tabs as ^I.  If this option is immediately followed\n", stderr);
    fputs("               by an integer, the integer sets the tab width.\n", stderr);
+   fputs("        -u:  Disable UTF-8 mode\n", stderr);
    fputs("        -v:  Do not interpret backspace formatting characters.\n", stderr);
    fputs("        -w:  Wrap lines.\n", stderr);
    fputs("        -z:  No gunzip-on-the-fly.\n", stderr);
@@ -108,6 +112,7 @@ void most_usage (void)
    fputs("        +d:  Allow file deletion.\n", stderr);
    fputs("        +s:  Secure Mode-- no edit, cd, shell, and reading files not\n", stderr);
    fputs("               already listed on the command line.\n", stderr);
+   fputs("        +u:  Enable UTF-8 mode.\n", stderr);
    fprintf(stderr, "\nExample: most -ct4 +82 keymap.c\n");
    fputs(" makes searches case sensitive, sets tabwidth to 4, and displays the file\n", stderr);
    fputs(" keymap.c starting at line 82.\n", stderr);
@@ -165,6 +170,12 @@ static void do_extended_switches(char *str)
 		case 's':
 		  Most_Secure_Mode = 1;
 		  break;
+		  
+		case 'U':
+		case 'u':
+		  Most_UTF8_Mode = 1;  /* +u */
+		  break;
+
 		default:
 		  fprintf(stderr,"%s invalid extended option %c ignored.\n",
 			  Most_Program, ch);
@@ -200,6 +211,9 @@ static void do_switches(char *str)
 	   case 'c':
 	     Most_Case_Sensitive = 1;
 	     break;
+	   case 'd':
+	   case 'D':
+	     Most_Show_Wrap_Marker = 0;
 	   case 's':
 	   case 'S':
 	     Most_S_Opt = 1; break;
@@ -212,7 +226,8 @@ static void do_switches(char *str)
 
 	   case 'K':		       /* Kanji option */
 	   case 'k':
-	     Most_K_Opt = 1; break;
+	     /* Most_K_Opt = 1; break; */
+	     break;
 
 	   case 'B':
 	   case 'b':
@@ -248,6 +263,11 @@ static void do_switches(char *str)
 	   case '1': assume_vt100 = 1;
 	     break;
 	     
+	   case 'u':
+	   case 'U':
+	     Most_UTF8_Mode = 0;       /* -u */
+	     break;
+
 	     /* Allow MOST_SWITCHES environment variable to contain + forms,
 	      * e.g., "-sn+d" or "-s -n +d"
 	      */
@@ -329,7 +349,8 @@ void most_initialize_most (void)
 
 static void do_most (char *file, int start)
 {
-   int piped, row, col;
+   int piped;
+   MOST_INT row, col;
 
    most_get_cdir(Most_C_Dir);
 
@@ -381,6 +402,22 @@ void most_exit_most (void)
 #endif
 }
 
+static void utf8_config (void)
+{
+   int utf8_mode = Most_UTF8_Mode;
+
+   utf8_mode = SLutf8_enable (-1);     /* returns 0 or 1 */
+   if (Most_UTF8_Mode == -1)
+     Most_UTF8_Mode = utf8_mode;
+   else if (utf8_mode != Most_UTF8_Mode)
+     {
+	if (utf8_mode == 1)
+	  (void) SLsmg_utf8_enable (0);   /* locale is UTF-8, but -u passed */
+	else
+	  (void) SLsmg_utf8_enable (1);   /* locale not UTF-8, but +u passed */
+     }
+}
+
 int most (int argc, char **argv)
 {
    char file[MAX_PATHLEN], *switches;
@@ -454,15 +491,7 @@ int most (int argc, char **argv)
    context = 0;
 
    SLtt_get_terminfo();
-#if SLANG_VERSION >= 20000
-#if 0
-   Most_UTF8_Mode = SLutf8_enable (1);
-   if (Most_UTF8_Mode)
-     {
-	fprintf (stderr, "UTF-8 Mode is in effect\n");
-     }
-#endif
-#endif
+   utf8_config ();
    SLtt_Ignore_Beep = 1;
    if (No_Colors) 
      SLtt_Use_Ansi_Colors = 0;
