@@ -1,5 +1,24 @@
 #ifndef _DAVIS_WINDOW_H_
 # define _DAVIS_WINDOW_H_
+/*
+ This file is part of MOST.
+
+ Copyright (c) 1991, 1999, 2002, 2005, 2006, 2007 John E. Davis
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA.
+*/
 # include "buffer.h"
 
 typedef struct _Most_Window_Type
@@ -56,7 +75,7 @@ extern void most_check_minibuffer(void);
 extern void most_free_window_buffer(void);
 extern void most_select_minibuffer(void);
 extern void most_exit_minibuffer(void);
-extern int most_read_from_minibuffer(char *, char *, unsigned int);
+extern int most_read_from_minibuffer(char *, char *, char *, unsigned int);
 extern void most_redraw_display(void);
 extern void most_one_window(void);
 extern void most_two_windows(void);
